"""
PyPG: A library for PPG (PhotoPlethysmoGram) processing

Documentation is available in the docstrings for now; online soon.


Modules
-----------
Using the modules requires an explicit import. For example:
``import pypg.filters``

::

filters
plots


Inspired By
-----------
SciPy: A scientific computing package for Python

.. seealso::

    https://github.com/scipy/scipy

"""

import pypg.filters
import pypg.plots
